import { Body, Controller, Headers, HttpCode, HttpStatus, Post, UseGuards } from '@nestjs/common';
import { ApiBearerAuth, ApiTags } from '@nestjs/swagger';
import { DEVICE_ID_HEADER } from '~/common/constants';
import { AuthenticatedUser } from '~/common/decorators';
import { AccessTokenGuard } from '~/common/guards';
import { ResponseFactory } from '~/core/utils';
import {
  CreateUnverifiedUserRequestDto,
  DisableBiometricRequestDto,
  EnableBiometricRequestDto,
  ForgetPasswordRequestDto,
  LoginRequestDto,
  SendForgetPasswordOtpRequestDto,
  SetEmailRequestDto,
  SetPasscodeRequestDto,
  VerifyUserRequestDto,
} from '../dtos/request';
import { SendForgetPasswordOtpResponseDto, SendRegisterOtpResponseDto } from '../dtos/response';
import { LoginResponseDto } from '../dtos/response/login.response.dto';
import { IJwtPayload } from '../interfaces';
import { AuthService } from '../services';

@Controller('auth')
@ApiTags('Auth')
@ApiBearerAuth()
export class AuthController {
  constructor(private readonly authService: AuthService) {}
  @Post('register/otp')
  async register(@Body() createUnverifiedUserDto: CreateUnverifiedUserRequestDto) {
    const phoneNumber = await this.authService.sendRegisterOtp(createUnverifiedUserDto);
    return ResponseFactory.data(new SendRegisterOtpResponseDto(phoneNumber));
  }

  @Post('register/verify')
  async verifyUser(@Body() verifyUserDto: VerifyUserRequestDto) {
    const [res, user] = await this.authService.verifyUser(verifyUserDto);
    return ResponseFactory.data(new LoginResponseDto(res, user));
  }

  @Post('register/set-email')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  async setEmail(@AuthenticatedUser() { sub }: IJwtPayload, @Body() setEmailDto: SetEmailRequestDto) {
    await this.authService.setEmail(sub, setEmailDto);
  }

  @Post('register/set-passcode')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  async setPasscode(@AuthenticatedUser() { sub }: IJwtPayload, @Body() { passcode }: SetPasscodeRequestDto) {
    await this.authService.setPasscode(sub, passcode);
  }

  @Post('biometric/enable')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  enableBiometric(@AuthenticatedUser() { sub }: IJwtPayload, @Body() enableBiometricDto: EnableBiometricRequestDto) {
    return this.authService.enableBiometric(sub, enableBiometricDto);
  }

  @Post('biometric/disable')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  disableBiometric(@AuthenticatedUser() { sub }: IJwtPayload, @Body() disableBiometricDto: DisableBiometricRequestDto) {
    return this.authService.disableBiometric(sub, disableBiometricDto);
  }

  @Post('forget-password/otp')
  async forgetPassword(@Body() sendForgetPasswordOtpDto: SendForgetPasswordOtpRequestDto) {
    const email = await this.authService.sendForgetPasswordOtp(sendForgetPasswordOtpDto);
    return ResponseFactory.data(new SendForgetPasswordOtpResponseDto(email));
  }

  @Post('forget-password/reset')
  @HttpCode(HttpStatus.NO_CONTENT)
  resetPassword(@Body() forgetPasswordDto: ForgetPasswordRequestDto) {
    return this.authService.verifyForgetPasswordOtp(forgetPasswordDto);
  }

  @Post('login')
  async login(@Body() loginDto: LoginRequestDto, @Headers(DEVICE_ID_HEADER) deviceId: string) {
    const [res, user] = await this.authService.login(loginDto, deviceId);
    return ResponseFactory.data(new LoginResponseDto(res, user));
  }
}
