import { DeepMocked, createMock } from '@golevelup/ts-jest';
import { HealthCheckResult, HealthCheckService, HttpHealthIndicator, TypeOrmHealthIndicator } from '@nestjs/terminus';
import { Test, TestingModule } from '@nestjs/testing';
import { HealthController } from './health.controller';

describe('HealthController', () => {
  let healthController: HealthController;
  let healthCheckServiceMock: DeepMocked<HealthCheckService>;
  let databaseHealthIndicatorMock: DeepMocked<TypeOrmHealthIndicator>;
  let httpHealthIndicatorMock: DeepMocked<HttpHealthIndicator>;

  const mockHealthCheckResult: HealthCheckResult = { status: 'ok', details: {} };

  beforeEach(async () => {
    // Mocks

    healthCheckServiceMock = createMock<HealthCheckService>({ check: jest.fn() });
    databaseHealthIndicatorMock = createMock<TypeOrmHealthIndicator>();
    httpHealthIndicatorMock = createMock<HttpHealthIndicator>();

    // Module
    const module: TestingModule = await Test.createTestingModule({
      controllers: [HealthController],
      providers: [
        { provide: HealthCheckService, useValue: healthCheckServiceMock },
        { provide: TypeOrmHealthIndicator, useValue: databaseHealthIndicatorMock },
        { provide: HttpHealthIndicator, useValue: httpHealthIndicatorMock },
      ],
    }).compile();

    healthController = module.get<HealthController>(HealthController);
  });

  it('should be defined', () => {
    expect(healthController).toBeDefined();
  });

  describe('With no detailed dependencies checks', () => {
    it('should return a health check result with no details', async () => {
      healthCheckServiceMock.check.mockResolvedValueOnce(mockHealthCheckResult);

      const result = await healthController.checkHealth();

      expect(result).toEqual(mockHealthCheckResult);
      expect(healthCheckServiceMock.check).toHaveBeenCalledTimes(1);
      expect(healthCheckServiceMock.check).toHaveBeenCalledWith([]);
    });
  });

  describe('With detailed dependencies checks', () => {
    it('should return a health check result with dependencies details', async () => {
      const expectedIndicatorsCnt = 1;
      healthCheckServiceMock.check.mockResolvedValueOnce(mockHealthCheckResult);

      const result = await healthController.checkHealthDetails();

      expect(result).toEqual(mockHealthCheckResult);
      expect(healthCheckServiceMock.check).toHaveBeenCalledTimes(1);
      expect(healthCheckServiceMock.check).toHaveBeenCalledWith(
        Array(expectedIndicatorsCnt).fill(expect.any(Function)),
      );
    });

    it('should use the expected health indicators to get the health result', async () => {
      const expectedIndicatorsCnt = 1;
      healthCheckServiceMock.check.mockImplementationOnce((indicators) => indicators as any);

      const indicators: any = await healthController.checkHealthDetails();
      expect(healthCheckServiceMock.check).toHaveBeenCalledWith(
        Array(expectedIndicatorsCnt).fill(expect.any(Function)),
      );

      // call each indicator to check the arguments passed to it
      for (let i = 0; i < expectedIndicatorsCnt; i++) {
        indicators[`${i}`]();
      }
      // database health indicator(s)
      expect(databaseHealthIndicatorMock.pingCheck).toHaveBeenCalledTimes(1);
      expect(databaseHealthIndicatorMock.pingCheck).toHaveBeenCalledWith('database');
    });
  });
});
