import { ApiProperty } from '@nestjs/swagger';
import {
  IsBoolean,
  IsDateString,
  IsEnum,
  IsNotEmpty,
  IsNumber,
  IsOptional,
  IsPositive,
  IsString,
  IsUUID,
  MaxLength,
} from 'class-validator';
import { i18nValidationMessage as i18n } from 'nestjs-i18n';
import { TaskFrequency } from '~/task/enums/task.frequency.enum';
const TEXT_LENGTH = 255;
const MAX_DECIMAL_PLACES = 3;
export class CreateTaskRequestDto {
  @ApiProperty()
  @IsString({ message: i18n('validation.IsString', { path: 'general', property: 'task.title' }) })
  @IsNotEmpty({ message: i18n('validation.IsNotEmpty', { path: 'general', property: 'task.title' }) })
  @MaxLength(TEXT_LENGTH, {
    message: i18n('validation.MaxLength', { path: 'general', property: 'task.title', length: TEXT_LENGTH }),
  })
  title!: string;

  @ApiProperty({ required: false })
  @IsString({ message: i18n('validation.IsString', { path: 'general', property: 'task.description' }) })
  @IsNotEmpty({ message: i18n('validation.IsNotEmpty', { path: 'general', property: 'task.description' }) })
  @MaxLength(TEXT_LENGTH, {
    message: i18n('validation.MaxLength', { path: 'general', property: 'task.description', length: TEXT_LENGTH }),
  })
  @IsOptional()
  description!: string;

  @IsDateString({}, { message: i18n('validation.IsDateString', { path: 'general', property: 'task.startDate' }) })
  @IsOptional()
  startDate: string = new Date().toISOString();

  @ApiProperty({ example: '2024-01-15' })
  @IsDateString({}, { message: i18n('validation.IsDateString', { path: 'general', property: 'task.dueDate' }) })
  dueDate!: string;

  @ApiProperty({ example: 100 })
  @IsNumber(
    { maxDecimalPlaces: MAX_DECIMAL_PLACES },
    { message: i18n('validation.IsNumber', { path: 'general', property: 'task.rewardAmount' }) },
  )
  @IsPositive({ message: i18n('validation.IsPositive', { path: 'general', property: 'task.rewardAmount' }) })
  rewardAmount!: number;

  @IsEnum(TaskFrequency, { message: i18n('validation.IsEnum', { path: 'general', property: 'task.frequency' }) })
  @IsOptional()
  frequency: TaskFrequency = TaskFrequency.ONE_TIME;

  @ApiProperty({ example: false })
  @IsBoolean({ message: i18n('validation.IsBoolean', { path: 'general', property: 'task.isProofRequired' }) })
  isProofRequired!: boolean;

  @ApiProperty({ example: 'e7b1b3b1-4b3b-4b3b-4b3b-4b3b4b3b4b3b' })
  @IsUUID('4', { message: i18n('validation.IsUUID', { path: 'general', property: 'task.imageId' }) })
  imageId!: string;

  @ApiProperty({ example: 'e7b1b3b1-4b3b-4b3b-4b3b-4b3b4b3b4b3b' })
  @IsUUID('4', { message: i18n('validation.IsUUID', { path: 'general', property: 'task.juniorId' }) })
  juniorId!: string;
}
