import {
  BaseEntity,
  Column,
  CreateDateColumn,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToOne,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { Document } from '~/document/entities';
import { Guardian } from '~/guardian/entities/guradian.entity';
import { Junior } from '~/junior/entities';
import { SubmissionStatus, TaskStatus } from '../enums';
import { TaskFrequency } from '../enums/task.frequency.enum';
import { TaskSubmission } from './task-submissions.entity';

@Entity('tasks')
export class Task extends BaseEntity {
  @PrimaryGeneratedColumn('uuid')
  id!: string;

  @Column({ type: 'varchar', length: 255, name: 'title' })
  title!: string;

  @Column({ type: 'varchar', length: 255, name: 'description' })
  description!: string;

  @Column({ type: 'decimal', name: 'reward_amount', precision: 12, scale: 3 })
  rewardAmount!: number;

  @Column({ type: 'uuid', name: 'image_id' })
  imageId!: string;

  @Column({ type: 'varchar', name: 'task_frequency' })
  taskFrequency!: TaskFrequency;

  @Column({ type: 'date', name: 'start_date' })
  startDate!: Date;

  @Column({ type: 'date', name: 'due_date' })
  dueDate!: Date;

  @Column({ type: 'boolean', name: 'is_proof_required' })
  isProofRequired!: boolean;

  @Column({ type: 'uuid', name: 'assigned_to_id' })
  assignedToId!: string;

  @Column({ type: 'uuid', name: 'assigned_by_id' })
  assignedById!: string;

  @ManyToOne(() => Document, (document) => document.tasks)
  @JoinColumn({ name: 'image_id' })
  image!: Document;

  @ManyToOne(() => Junior, (junior) => junior.tasks, { onDelete: 'CASCADE' })
  @JoinColumn({ name: 'assigned_to_id' })
  assignedTo!: Junior;

  @ManyToOne(() => Guardian, (guardian) => guardian.tasks)
  @JoinColumn({ name: 'assigned_by_id' })
  assignedBy!: Guardian;

  @OneToOne(() => TaskSubmission, (submission) => submission.task, { cascade: true })
  submission?: TaskSubmission;

  @CreateDateColumn({ type: 'timestamp', name: 'created_at', default: () => 'CURRENT_TIMESTAMP' })
  createdAt!: Date;

  @UpdateDateColumn({ type: 'timestamp', name: 'updated_at', default: () => 'CURRENT_TIMESTAMP' })
  updatedAt!: Date;

  get status(): string {
    if (new Date(this.dueDate) < new Date()) {
      return TaskStatus.COMPLETED;
    }

    if (this.submission && this.submission.status === SubmissionStatus.APPROVED) {
      return TaskStatus.COMPLETED;
    }

    if (this.submission && this.submission.status !== SubmissionStatus.APPROVED) {
      return TaskStatus.IN_PROGRESS;
    }

    return TaskStatus.PENDING;
  }
}
